<?php
/**
 * Searcher for Elementor
 * Customizable search bar for Elementor editor
 * Exclusively on https://1.envato.market/searcher-elementor
 *
 * @encoding        UTF-8
 * @version         1.0.2
 * @copyright       (C) 2018 - 2022 Merkulove ( https://merkulov.design/ ). All rights reserved.
 * @license         Envato License https://1.envato.market/KYbje
 * @contributors    Nemirovskiy Vitaliy (nemirovskiyvitaliy@gmail.com), Dmitry Merkulov (dmitry@merkulov.design), Cherviakov Vlad (vladchervjakov@gmail.com)
 * @support         help@merkulov.design
 **/

namespace Merkulove\SearcherElementor;

/** Exit if accessed directly. */
if ( ! defined( 'ABSPATH' ) ) {
	header( 'Status: 403 Forbidden' );
	header( 'HTTP/1.1 403 Forbidden' );
	exit;
}

/**
 * SINGLETON: Caster class contain main plugin logic.
 *
 * @since 1.0.0
 *
 **/
final class Caster {

	/**
	 * The one true Caster.
	 *
     * @since 1.0.0
     * @access private
	 * @var Caster
	 **/
	private static $instance;

    /**
     * Setup the plugin.
     *
     * @since 1.0.0
     * @access public
     *
     * @return void
     **/
    public function setup() {

        /** Define hooks that runs on both the front-end as well as the dashboard. */
        $this->both_hooks();

        /** Define public hooks. */
        $this->public_hooks();

        /** Define admin hooks. */
        $this->admin_hooks();

    }

    /**
     * Define hooks that runs on both the front-end as well as the dashboard.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function both_hooks() {

        /** Hooks which should work everywhere in backend and frontend. */

        add_action( 'wp_ajax_mdp_async_search', [ $this, 'get_search_results' ] );
        add_action( 'wp_ajax_nopriv_mdp_async_search', [ $this, 'get_search_results' ] );

    }

    /**
     * Register all of the hooks related to the public-facing functionality.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function public_hooks() {

        /** Work only on frontend area. */
        if ( is_admin() ) { return; }

         add_action( 'pre_get_posts', [$this, 'search_filter'], 9 );

        /** Public hooks and filters. */

    }

    /**
     * Register all of the hooks related to the admin area functionality.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function admin_hooks() {

        /** Work only in admin area. */
        if ( ! is_admin() ) { return; }

        /** Admin hooks and filters here. */

    }

	/**
	 * This method used in register_activation_hook
	 * Everything written here will be done during plugin activation.
	 *
	 * @since 1.0.0
	 * @access public
	 */
	public function activation_hook() {

		/** Activation hook */

	}

    /**
     * Filters search by post type.
     * @since 1.0.0
     * @access public
     *
     * @return void
     **/
    public function search_filter( $query ) {
        if ( $query->is_search() ) {
            if ( isset( $_GET['mdp_post_type'] ) ) {
                $post_type = array_map( 'sanitize_text_field', $_GET['mdp_post_type'] );
                $query->set( 'post_type', $post_type );
            }
        }

        return $query;
    }

    /**
     * This method returns search results
     *
     * @since 1.0.0
     * @access public
     */
    public function get_search_results() {
        if ( empty( $_GET['mdp_searcher_nonce'] ) ||
            ! wp_verify_nonce( $_GET['mdp_searcher_nonce'], 'mdp-searcher-elementor-nonce' ) ) { return; }

        $search_result = [];

        $search_value = sanitize_text_field( $_GET['mdp_search_value'] );

        $post_types = explode( ',', sanitize_text_field( $_GET['mdp_post_types'] ) );

        $results_quantity = sanitize_text_field( $_GET['mdp_results_quantity'] );

        $query = new \WP_Query( [
            'posts_per_page' => $results_quantity,
            's' => $search_value,
            'post_type' => $post_types,
            'post_status' => 'publish'
        ] );

        if ( $query->have_posts() ) {
            while ( $query->have_posts() ) {
                $query->the_post();
                $post_image = sprintf(
            '<div class="mdp-searcher-elementor-search-img-container"><img src="%s" alt=""></div>',
                    esc_url( get_the_post_thumbnail_url() )
                );
                $search_result[] = sprintf(
                    "<div class='mdp-searcher-elementor-result-block'>
                                <a class='mdp-searcher-elementor-search-result' href='%s'>
                                    %s
                                    <div class='mdp-searcher-elementor-search-content'>
                                        <div class='mdp-searcher-elementor-search-post-title'>%s</div>
                                        <div class='mdp-searcher-elementor-search-post-excerpt'>%s</div>
                                    </div>
                                </a>
                            </div>",
                    esc_url( get_permalink() ),
                    !empty( get_the_post_thumbnail_url() ) ?
                        $post_image :
                    '',
                    the_title( '', '', false ),
                    strlen( get_the_excerpt() ) > 20 ?
                        substr( esc_html( get_the_excerpt() ), 0, 20 ) . '...' :
                        get_the_excerpt()
                );
            }
            wp_send_json_success( $search_result );
        } else {
            wp_send_json_success( [ 'is_empty' => true ] );
        }

    }

	/**
	 * Main Caster Instance.
	 * Insures that only one instance of Caster exists in memory at any one time.
	 *
	 * @static
     * @since 1.0.0
     * @access public
     *
	 * @return Caster
	 **/
	public static function get_instance() {

		if ( ! isset( self::$instance ) && ! ( self::$instance instanceof self ) ) {

			self::$instance = new self;

		}

		return self::$instance;

	}

}
